@@ -3,13 +3,13 @@ package component
 import (
 	"fmt"
 	"sort"
+	"strconv"
 
 	"k8s.io/api/policy/v1beta1"
 
 	"github.com/3scale/3scale-operator/pkg/helper"
 	appsv1 "github.com/openshift/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
-	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 )
@@ -92,11 +92,24 @@ const (
 )
 
 const (
-	SystemSidekiqName = "system-sidekiq"
+	SystemSidekiqName       = "system-sidekiq"
+	SystemAppDeploymentName = "system-app"
+
+	SystemAppMasterContainerName    = "system-master"
+	SystemAppProviderContainerName  = "system-provider"
+	SystemAppDeveloperContainerName = "system-developer"
 )
 
 const (
-	SystemSidekiqMetricsPort = 9394
+	SystemAppPrometheusExporterPortEnvVarName     = "PROMETHEUS_EXPORTER_PORT"
+	SystemSidekiqPrometheusExporterPortEnvVarName = "PROMETHEUS_EXPORTER_PORT"
+	SystemSidekiqMetricsPort                      = 9394
+	SystemAppMasterContainerPrometheusPort        = 9395
+	SystemAppProviderContainerPrometheusPort      = 9396
+	SystemAppDeveloperContainerPrometheusPort     = 9394
+	SystemAppMasterContainerMetricsPortName       = "master-metrics"
+	SystemAppProviderContainerMetricsPortName     = "prov-metrics"
+	SystemAppDeveloperContainerMetricsPortName    = "dev-metrics"
 )
 
 type System struct {
@@ -241,6 +254,41 @@ func (system *System) buildSystemBaseEnv() []v1.EnvVar {
 	return result
 }
 
+func (system *System) buildAppMasterContainerEnv() []v1.EnvVar {
+	result := system.buildSystemBaseEnv()
+	if system.Options.AppMetrics {
+		result = append(result, helper.EnvVarFromValue(SystemAppPrometheusExporterPortEnvVarName, strconv.Itoa(SystemAppMasterContainerPrometheusPort)))
+	}
+
+	return result
+}
+
+func (system *System) buildAppProviderContainerEnv() []v1.EnvVar {
+	result := system.buildSystemBaseEnv()
+	if system.Options.AppMetrics {
+		result = append(result, helper.EnvVarFromValue(SystemAppPrometheusExporterPortEnvVarName, strconv.Itoa(SystemAppProviderContainerPrometheusPort)))
+	}
+	return result
+}
+
+func (system *System) buildAppDeveloperContainerEnv() []v1.EnvVar {
+	result := system.buildSystemBaseEnv()
+	if system.Options.AppMetrics {
+		result = append(result, helper.EnvVarFromValue(SystemAppPrometheusExporterPortEnvVarName, strconv.Itoa(SystemAppDeveloperContainerPrometheusPort)))
+	}
+
+	return result
+}
+
+func (system *System) buildSystemSidekiqContainerEnv() []v1.EnvVar {
+	result := system.buildSystemBaseEnv()
+	if system.Options.SideKiqMetrics {
+		result = append(result, helper.EnvVarFromValue(SystemSidekiqPrometheusExporterPortEnvVarName, strconv.Itoa(SystemSidekiqMetricsPort)))
+	}
+
+	return result
+}
+
 func (system *System) buildSystemAppPreHookEnv() []v1.EnvVar {
 	result := []v1.EnvVar{}
 	baseEnv := system.buildSystemBaseEnv()
@@ -248,6 +296,17 @@ func (system *System) buildSystemAppPreHookEnv() []v1.EnvVar {
 	result = append(result,
 		helper.EnvVarFromSecret("MASTER_ACCESS_TOKEN", SystemSecretSystemSeedSecretName, SystemSecretSystemSeedMasterAccessTokenFieldName),
 	)
+	if system.Options.IncludeOracleOptionalSettings {
+		result = append(result, helper.EnvVarFromSecretOptional("ORACLE_SYSTEM_PASSWORD", SystemSecretSystemDatabaseSecretName, "ORACLE_SYSTEM_PASSWORD"))
+	}
+	return result
+}
+
+func (system *System) buildSystemAppPostHookEnv() []v1.EnvVar {
+	var result []v1.EnvVar
+	if system.Options.IncludeOracleOptionalSettings {
+		result = append(result, helper.EnvVarFromSecretOptional("ORACLE_SYSTEM_PASSWORD", SystemSecretSystemDatabaseSecretName, "ORACLE_SYSTEM_PASSWORD"))
+	}
 	return result
 }
 
@@ -344,30 +403,6 @@ func (system *System) EventsHookSecret() *v1.Secret {
 	}
 }
 
-func (system *System) RedisSecret() *v1.Secret {
-	return &v1.Secret{
-		TypeMeta: metav1.TypeMeta{
-			APIVersion: "v1",
-			Kind:       "Secret",
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Name:   SystemSecretSystemRedisSecretName,
-			Labels: system.Options.CommonLabels,
-		},
-		StringData: map[string]string{
-			SystemSecretSystemRedisURLFieldName:                system.Options.RedisURL,
-			SystemSecretSystemRedisSentinelHosts:               *system.Options.RedisSentinelHosts,
-			SystemSecretSystemRedisSentinelRole:                *system.Options.RedisSentinelRole,
-			SystemSecretSystemRedisMessageBusRedisURLFieldName: *system.Options.MessageBusRedisURL,
-			SystemSecretSystemRedisMessageBusSentinelHosts:     *system.Options.MessageBusRedisSentinelHosts,
-			SystemSecretSystemRedisMessageBusSentinelRole:      *system.Options.MessageBusRedisSentinelRole,
-			SystemSecretSystemRedisNamespace:                   *system.Options.RedisNamespace,
-			SystemSecretSystemRedisMessageBusRedisNamespace:    *system.Options.MessageBusRedisNamespace,
-		},
-		Type: v1.SecretTypeOpaque,
-	}
-}
-
 func (system *System) AppSecret() *v1.Secret {
 	return &v1.Secret{
 		TypeMeta: metav1.TypeMeta{
@@ -478,7 +513,7 @@ func (system *System) AppDeploymentConfig() *appsv1.DeploymentConfig {
 			APIVersion: "apps.openshift.io/v1",
 		},
 		ObjectMeta: metav1.ObjectMeta{
-			Name:   "system-app",
+			Name:   SystemAppDeploymentName,
 			Labels: system.Options.CommonAppLabels,
 		},
 		Spec: appsv1.DeploymentConfigSpec{
@@ -503,14 +538,15 @@ func (system *System) AppDeploymentConfig() *appsv1.DeploymentConfig {
 							// but changing that probably has some implications during an upgrade process of the product
 							Command:       []string{"bash", "-c", "bundle exec rake boot openshift:deploy"},
 							Env:           system.buildSystemAppPreHookEnv(),
-							ContainerName: "system-master",
+							ContainerName: SystemAppMasterContainerName,
 							Volumes:       system.volumeNamesForSystemAppPreHookPod()},
 					},
 					Post: &appsv1.LifecycleHook{
 						FailurePolicy: appsv1.LifecycleHookFailurePolicyAbort,
 						ExecNewPod: &appsv1.ExecNewPodHook{
 							Command:       []string{"bash", "-c", "bundle exec rake boot openshift:post_deploy"},
-							ContainerName: "system-master"}}},
+							Env:           system.buildSystemAppPostHookEnv(),
+							ContainerName: SystemAppMasterContainerName}}},
 			},
 			MinReadySeconds: 0,
 			Triggers: appsv1.DeploymentTriggerPolicies{
@@ -520,13 +556,13 @@ func (system *System) AppDeploymentConfig() *appsv1.DeploymentConfig {
 					Type: appsv1.DeploymentTriggerOnImageChange,
 					ImageChangeParams: &appsv1.DeploymentTriggerImageChangeParams{
 						Automatic:      true,
-						ContainerNames: []string{"system-provider", "system-developer", "system-master"},
+						ContainerNames: []string{SystemAppProviderContainerName, SystemAppDeveloperContainerName, SystemAppMasterContainerName},
 						From: v1.ObjectReference{
 							Kind: "ImageStreamTag",
 							Name: fmt.Sprintf("amp-system:%s", system.Options.ImageTag)}}},
 			},
 			Replicas: *system.Options.AppReplicas,
-			Selector: map[string]string{"deploymentConfig": "system-app"},
+			Selector: map[string]string{"deploymentConfig": SystemAppDeploymentName},
 			Template: &v1.PodTemplateSpec{
 				ObjectMeta: metav1.ObjectMeta{
 					Labels: system.Options.AppPodTemplateLabels,
@@ -537,17 +573,11 @@ func (system *System) AppDeploymentConfig() *appsv1.DeploymentConfig {
 					Volumes:     system.appPodVolumes(),
 					Containers: []v1.Container{
 						v1.Container{
-							Name:  "system-master",
-							Image: "amp-system:latest",
-							Args:  []string{"env", "TENANT_MODE=master", "PORT=3002", "container-entrypoint", "bundle", "exec", "unicorn", "-c", "config/unicorn.rb"},
-							Ports: []v1.ContainerPort{
-								v1.ContainerPort{
-									Name:          "master",
-									HostPort:      0,
-									ContainerPort: 3002,
-									Protocol:      v1.ProtocolTCP},
-							},
-							Env:          system.buildSystemBaseEnv(),
+							Name:         SystemAppMasterContainerName,
+							Image:        "amp-system:latest",
+							Args:         []string{"env", "TENANT_MODE=master", "PORT=3002", "container-entrypoint", "bundle", "exec", "unicorn", "-c", "config/unicorn.rb"},
+							Ports:        system.appMasterPorts(),
+							Env:          system.buildAppMasterContainerEnv(),
 							Resources:    *system.Options.AppMasterContainerResourceRequirements,
 							VolumeMounts: system.appMasterContainerVolumeMounts(),
 							LivenessProbe: &v1.Probe{
@@ -586,17 +616,11 @@ func (system *System) AppDeploymentConfig() *appsv1.DeploymentConfig {
 							StdinOnce:       false,
 							TTY:             false,
 						}, v1.Container{
-							Name:  "system-provider",
-							Image: "amp-system:latest",
-							Args:  []string{"env", "TENANT_MODE=provider", "PORT=3000", "container-entrypoint", "bundle", "exec", "unicorn", "-c", "config/unicorn.rb"},
-							Ports: []v1.ContainerPort{
-								v1.ContainerPort{
-									Name:          "provider",
-									HostPort:      0,
-									ContainerPort: 3000,
-									Protocol:      v1.ProtocolTCP},
-							},
-							Env:          system.buildSystemBaseEnv(),
+							Name:         SystemAppProviderContainerName,
+							Image:        "amp-system:latest",
+							Args:         []string{"env", "TENANT_MODE=provider", "PORT=3000", "container-entrypoint", "bundle", "exec", "unicorn", "-c", "config/unicorn.rb"},
+							Ports:        system.appProviderPorts(),
+							Env:          system.buildAppProviderContainerEnv(),
 							Resources:    *system.Options.AppProviderContainerResourceRequirements,
 							VolumeMounts: system.appProviderContainerVolumeMounts(),
 							LivenessProbe: &v1.Probe{
@@ -635,17 +659,11 @@ func (system *System) AppDeploymentConfig() *appsv1.DeploymentConfig {
 							StdinOnce:       false,
 							TTY:             false,
 						}, v1.Container{
-							Name:  "system-developer",
-							Image: "amp-system:latest",
-							Args:  []string{"env", "PORT=3001", "container-entrypoint", "bundle", "exec", "unicorn", "-c", "config/unicorn.rb"},
-							Ports: []v1.ContainerPort{
-								v1.ContainerPort{
-									Name:          "developer",
-									HostPort:      0,
-									ContainerPort: 3001,
-									Protocol:      v1.ProtocolTCP},
-							},
-							Env:          system.buildSystemBaseEnv(),
+							Name:         SystemAppDeveloperContainerName,
+							Image:        "amp-system:latest",
+							Args:         []string{"env", "PORT=3001", "container-entrypoint", "bundle", "exec", "unicorn", "-c", "config/unicorn.rb"},
+							Ports:        system.appDeveloperPorts(),
+							Env:          system.buildAppDeveloperContainerEnv(),
 							Resources:    *system.Options.AppDeveloperContainerResourceRequirements,
 							VolumeMounts: system.appDeveloperContainerVolumeMounts(),
 							LivenessProbe: &v1.Probe{
@@ -805,7 +823,7 @@ func (system *System) SidekiqDeploymentConfig() *appsv1.DeploymentConfig {
 							Name:            SystemSidekiqName,
 							Image:           "amp-system:latest",
 							Args:            []string{"rake", "sidekiq:worker", "RAILS_MAX_THREADS=25"},
-							Env:             system.buildSystemBaseEnv(),
+							Env:             system.buildSystemSidekiqContainerEnv(),
 							Resources:       *system.Options.SidekiqContainerResourceRequirements,
 							VolumeMounts:    system.sidekiqContainerVolumeMounts(),
 							ImagePullPolicy: v1.PullIfNotPresent,
@@ -874,6 +892,11 @@ func (system *System) sidekiqContainerVolumeMounts() []v1.VolumeMount {
 }
 
 func (system *System) SharedStorage() *v1.PersistentVolumeClaim {
+	volName := ""
+	if system.Options.PvcFileStorageOptions.VolumeName != nil {
+		volName = *system.Options.PvcFileStorageOptions.VolumeName
+	}
+
 	return &v1.PersistentVolumeClaim{
 		TypeMeta: metav1.TypeMeta{
 			APIVersion: "v1",
@@ -885,12 +908,13 @@ func (system *System) SharedStorage() *v1.PersistentVolumeClaim {
 		},
 		Spec: v1.PersistentVolumeClaimSpec{
 			StorageClassName: system.Options.PvcFileStorageOptions.StorageClass,
+			VolumeName:       volName,
 			AccessModes: []v1.PersistentVolumeAccessMode{
 				v1.ReadWriteMany,
 			},
 			Resources: v1.ResourceRequirements{
 				Requests: v1.ResourceList{
-					v1.ResourceStorage: resource.MustParse("100Mi"),
+					v1.ResourceStorage: system.Options.PvcFileStorageOptions.StorageRequests,
 				},
 			},
 		},
@@ -916,7 +940,7 @@ func (system *System) ProviderService() *v1.Service {
 					TargetPort: intstr.FromString("provider"),
 				},
 			},
-			Selector: map[string]string{"deploymentConfig": "system-app"},
+			Selector: map[string]string{"deploymentConfig": SystemAppDeploymentName},
 		},
 	}
 }
@@ -940,7 +964,7 @@ func (system *System) MasterService() *v1.Service {
 					TargetPort: intstr.FromString("master"),
 				},
 			},
-			Selector: map[string]string{"deploymentConfig": "system-app"},
+			Selector: map[string]string{"deploymentConfig": SystemAppDeploymentName},
 		},
 	}
 }
@@ -964,7 +988,7 @@ func (system *System) DeveloperService() *v1.Service {
 					TargetPort: intstr.FromString("developer"),
 				},
 			},
-			Selector: map[string]string{"deploymentConfig": "system-app"},
+			Selector: map[string]string{"deploymentConfig": SystemAppDeploymentName},
 		},
 	}
 }
@@ -1211,7 +1235,7 @@ func (system *System) AppPodDisruptionBudget() *v1beta1.PodDisruptionBudget {
 		},
 		Spec: v1beta1.PodDisruptionBudgetSpec{
 			Selector: &metav1.LabelSelector{
-				MatchLabels: map[string]string{"deploymentConfig": "system-app"},
+				MatchLabels: map[string]string{"deploymentConfig": SystemAppDeploymentName},
 			},
 			MaxUnavailable: &intstr.IntOrString{IntVal: PDB_MAX_UNAVAILABLE_POD_NUMBER},
 		},
@@ -1246,3 +1270,39 @@ func (system *System) sideKiqPorts() []v1.ContainerPort {
 
 	return ports
 }
+
+func (system *System) appMasterPorts() []v1.ContainerPort {
+	var ports []v1.ContainerPort
+
+	ports = append(ports, v1.ContainerPort{Name: "master", HostPort: 0, ContainerPort: 3002, Protocol: v1.ProtocolTCP})
+
+	if system.Options.AppMetrics {
+		ports = append(ports, v1.ContainerPort{Name: SystemAppMasterContainerMetricsPortName, ContainerPort: SystemAppMasterContainerPrometheusPort, Protocol: v1.ProtocolTCP})
+	}
+
+	return ports
+}
+
+func (system *System) appProviderPorts() []v1.ContainerPort {
+	var ports []v1.ContainerPort
+
+	ports = append(ports, v1.ContainerPort{Name: "provider", HostPort: 0, ContainerPort: 3000, Protocol: v1.ProtocolTCP})
+
+	if system.Options.AppMetrics {
+		ports = append(ports, v1.ContainerPort{Name: SystemAppProviderContainerMetricsPortName, ContainerPort: SystemAppProviderContainerPrometheusPort, Protocol: v1.ProtocolTCP})
+	}
+
+	return ports
+}
+
+func (system *System) appDeveloperPorts() []v1.ContainerPort {
+	var ports []v1.ContainerPort
+
+	ports = append(ports, v1.ContainerPort{Name: "developer", HostPort: 0, ContainerPort: 3001, Protocol: v1.ProtocolTCP})
+
+	if system.Options.AppMetrics {
+		ports = append(ports, v1.ContainerPort{Name: SystemAppDeveloperContainerMetricsPortName, ContainerPort: SystemAppDeveloperContainerPrometheusPort, Protocol: v1.ProtocolTCP})
+	}
+
+	return ports
+}
