@@ -65,6 +65,10 @@ func (s *SystemOptionsProvider) GetSystemOptions() (*component.SystemOptions, er
 	s.setReplicas()
 
 	s.options.SideKiqMetrics = true
+	s.options.AppMetrics = true
+	s.options.IncludeOracleOptionalSettings = true
+
+	s.options.Namespace = s.namespace
 
 	err = s.options.Validate()
 	if err != nil {
@@ -90,11 +94,6 @@ func (s *SystemOptionsProvider) setSecretBasedOptions() error {
 		return fmt.Errorf("unable to create System Event Hooks secret options - %s", err)
 	}
 
-	err = s.setSystemRedisOptions()
-	if err != nil {
-		return fmt.Errorf("unable to create System Redis secret options - %s", err)
-	}
-
 	err = s.setSystemAppOptions()
 	if err != nil {
 		return fmt.Errorf("unable to create System App secret options - %s", err)
@@ -175,77 +174,6 @@ func (s *SystemOptionsProvider) setSystemEventHookOptions() error {
 	return nil
 }
 
-func (s *SystemOptionsProvider) setSystemRedisOptions() error {
-	val, err := s.secretSource.FieldValue(
-		component.SystemSecretSystemRedisSecretName,
-		component.SystemSecretSystemRedisURLFieldName,
-		component.DefaultSystemRedisURL())
-	if err != nil {
-		return err
-	}
-	s.options.RedisURL = val
-
-	cases := []struct {
-		field       **string
-		secretName  string
-		secretField string
-		defValue    string
-	}{
-		{
-			&s.options.RedisSentinelHosts,
-			component.SystemSecretSystemRedisSecretName,
-			component.SystemSecretSystemRedisSentinelHosts,
-			component.DefaultSystemRedisSentinelHosts(),
-		},
-		{
-			&s.options.RedisSentinelRole,
-			component.SystemSecretSystemRedisSecretName,
-			component.SystemSecretSystemRedisSentinelRole,
-			component.DefaultSystemRedisSentinelRole(),
-		},
-		{
-			&s.options.MessageBusRedisURL,
-			component.SystemSecretSystemRedisSecretName,
-			component.SystemSecretSystemRedisMessageBusRedisURLFieldName,
-			component.DefaultSystemMessageBusRedisURL(),
-		},
-		{
-			&s.options.MessageBusRedisSentinelHosts,
-			component.SystemSecretSystemRedisSecretName,
-			component.SystemSecretSystemRedisMessageBusSentinelHosts,
-			component.DefaultSystemMessageBusRedisSentinelHosts(),
-		},
-		{
-			&s.options.MessageBusRedisSentinelRole,
-			component.SystemSecretSystemRedisSecretName,
-			component.SystemSecretSystemRedisMessageBusSentinelRole,
-			component.DefaultSystemMessageBusRedisSentinelRole(),
-		},
-		{
-			&s.options.RedisNamespace,
-			component.SystemSecretSystemRedisSecretName,
-			component.SystemSecretSystemRedisNamespace,
-			component.DefaultSystemRedisNamespace(),
-		},
-		{
-			&s.options.MessageBusRedisNamespace,
-			component.SystemSecretSystemRedisSecretName,
-			component.SystemSecretSystemRedisMessageBusRedisNamespace,
-			component.DefaultSystemMessageBusRedisNamespace(),
-		},
-	}
-
-	for _, option := range cases {
-		val, err := s.secretSource.FieldValue(option.secretName, option.secretField, option.defValue)
-		if err != nil {
-			return err
-		}
-		*option.field = &val
-	}
-
-	return nil
-}
-
 func (s *SystemOptionsProvider) setSystemAppOptions() error {
 	val, err := s.secretSource.FieldValue(
 		component.SystemSecretSystemAppSecretName,
@@ -430,6 +358,25 @@ func (s *SystemOptionsProvider) setResourceRequirementsOptions() {
 		s.options.SidekiqContainerResourceRequirements = &v1.ResourceRequirements{}
 		s.options.SphinxContainerResourceRequirements = &v1.ResourceRequirements{}
 	}
+
+	// DeploymentConfig-level ResourceRequirements CR fields have priority over
+	// spec.resourceRequirementsEnabled, overwriting that setting when they are
+	// defined
+	if s.apimanager.Spec.System.AppSpec.MasterContainerResources != nil {
+		s.options.AppMasterContainerResourceRequirements = s.apimanager.Spec.System.AppSpec.MasterContainerResources
+	}
+	if s.apimanager.Spec.System.AppSpec.ProviderContainerResources != nil {
+		s.options.AppProviderContainerResourceRequirements = s.apimanager.Spec.System.AppSpec.ProviderContainerResources
+	}
+	if s.apimanager.Spec.System.AppSpec.DeveloperContainerResources != nil {
+		s.options.AppDeveloperContainerResourceRequirements = s.apimanager.Spec.System.AppSpec.DeveloperContainerResources
+	}
+	if s.apimanager.Spec.System.SidekiqSpec.Resources != nil {
+		s.options.SidekiqContainerResourceRequirements = s.apimanager.Spec.System.SidekiqSpec.Resources
+	}
+	if s.apimanager.Spec.System.SphinxSpec.Resources != nil {
+		s.options.SphinxContainerResourceRequirements = s.apimanager.Spec.System.SphinxSpec.Resources
+	}
 }
 
 func (s *SystemOptionsProvider) setNodeAffinityAndTolerationsOptions() {
@@ -451,14 +398,22 @@ func (s *SystemOptionsProvider) setFileStorageOptions() {
 	} else {
 		// default to PVC
 		var storageClassName *string
+		var volumeName *string
+		storageRequests := component.DefaultSharedStorageResources()
 		if s.apimanager.Spec.System != nil &&
 			s.apimanager.Spec.System.FileStorageSpec != nil &&
 			s.apimanager.Spec.System.FileStorageSpec.PVC != nil {
 			storageClassName = s.apimanager.Spec.System.FileStorageSpec.PVC.StorageClassName
+			volumeName = s.apimanager.Spec.System.FileStorageSpec.PVC.VolumeName
+			if s.apimanager.Spec.System.FileStorageSpec.PVC.Resources != nil {
+				storageRequests = s.apimanager.Spec.System.FileStorageSpec.PVC.Resources.Requests
+			}
 		}
 
 		s.options.PvcFileStorageOptions = &component.PVCFileStorageOptions{
-			StorageClass: storageClassName,
+			StorageClass:    storageClassName,
+			VolumeName:      volumeName,
+			StorageRequests: storageRequests,
 		}
 	}
 }
