@@ -7,6 +7,7 @@ import (
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/product"
 	appsv1alpha1 "github.com/3scale/3scale-operator/pkg/apis/apps/v1alpha1"
+
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
 	v1 "k8s.io/api/core/v1"
@@ -43,9 +44,9 @@ func testSystemAppPodTemplateLabels() map[string]string {
 		"threescale_component_element": "app",
 		"com.redhat.component-name":    "system-app",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.9",
+		"com.redhat.component-version": "2.10",
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.9",
+		"com.redhat.product-version":   "2.10",
 		"deploymentConfig":             "system-app",
 	}
 }
@@ -65,9 +66,9 @@ func testSystemSidekiqPodTemplateLabels() map[string]string {
 		"threescale_component_element": "sidekiq",
 		"com.redhat.component-name":    "system-sidekiq",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.9",
+		"com.redhat.component-version": "2.10",
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.9",
+		"com.redhat.product-version":   "2.10",
 		"deploymentConfig":             "system-sidekiq",
 	}
 }
@@ -111,9 +112,9 @@ func testSystemSphinxPodTemplateLabels() map[string]string {
 		"threescale_component_element": "sphinx",
 		"com.redhat.component-name":    "system-sphinx",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.9",
+		"com.redhat.component-version": "2.10",
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.9",
+		"com.redhat.product-version":   "2.10",
 		"deploymentConfig":             "system-sphinx",
 	}
 }
@@ -158,6 +159,71 @@ func testSystemSphinxTolerations() []v1.Toleration {
 	return getTestTolerations("system-sphinx")
 }
 
+func testSystemMasterContainerCustomResourceRequirements() *v1.ResourceRequirements {
+	return &v1.ResourceRequirements{
+		Limits: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("111m"),
+			v1.ResourceMemory: resource.MustParse("222Mi"),
+		},
+		Requests: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("333m"),
+			v1.ResourceMemory: resource.MustParse("444Mi"),
+		},
+	}
+}
+
+func testSystemProviderContainerCustomResourceRequirements() *v1.ResourceRequirements {
+	return &v1.ResourceRequirements{
+		Limits: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("222m"),
+			v1.ResourceMemory: resource.MustParse("333Mi"),
+		},
+		Requests: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("444m"),
+			v1.ResourceMemory: resource.MustParse("555Mi"),
+		},
+	}
+}
+
+func testSystemDeveloperContainerCustomResourceRequirements() *v1.ResourceRequirements {
+	return &v1.ResourceRequirements{
+		Limits: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("666m"),
+			v1.ResourceMemory: resource.MustParse("777Mi"),
+		},
+		Requests: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("888m"),
+			v1.ResourceMemory: resource.MustParse("999Mi"),
+		},
+	}
+}
+
+func testSystemSidekiqCustomResourceRequirements() *v1.ResourceRequirements {
+	return &v1.ResourceRequirements{
+		Limits: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("842m"),
+			v1.ResourceMemory: resource.MustParse("253Mi"),
+		},
+		Requests: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("294m"),
+			v1.ResourceMemory: resource.MustParse("195Mi"),
+		},
+	}
+}
+
+func testSystemSphinxCustomResourceRequirements() *v1.ResourceRequirements {
+	return &v1.ResourceRequirements{
+		Limits: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("123m"),
+			v1.ResourceMemory: resource.MustParse("456Mi"),
+		},
+		Requests: v1.ResourceList{
+			v1.ResourceCPU:    resource.MustParse("789m"),
+			v1.ResourceMemory: resource.MustParse("346Mi"),
+		},
+	}
+}
+
 func basicApimanagerSpecTestSystemOptions() *appsv1alpha1.APIManager {
 	tmpSystemAppReplicas := systemAppReplicas
 	tmpSystemSideKiqReplicas := systemSidekiqReplicas
@@ -198,20 +264,6 @@ func getEventHookSecret() *v1.Secret {
 	return GetTestSecret(namespace, component.SystemSecretSystemEventsHookSecretName, data)
 }
 
-func getSystemRedisSecret() *v1.Secret {
-	data := map[string]string{
-		component.SystemSecretSystemRedisNamespace:                   "systemRedis",
-		component.SystemSecretSystemRedisURLFieldName:                "redis://system1:6379",
-		component.SystemSecretSystemRedisSentinelHosts:               "someHosts1",
-		component.SystemSecretSystemRedisSentinelRole:                "someRole1",
-		component.SystemSecretSystemRedisMessageBusRedisNamespace:    "mbus",
-		component.SystemSecretSystemRedisMessageBusSentinelHosts:     "someHosts2",
-		component.SystemSecretSystemRedisMessageBusSentinelRole:      "someRole2",
-		component.SystemSecretSystemRedisMessageBusRedisURLFieldName: "redis://system2:6379",
-	}
-	return GetTestSecret(namespace, component.SystemSecretSystemRedisSecretName, data)
-}
-
 func getSystemAppSecret() *v1.Secret {
 	data := map[string]string{
 		component.SystemSecretSystemAppSecretKeyBaseFieldName: "somePassword234",
@@ -243,13 +295,6 @@ func getSystemMasterApicastSecret() *v1.Secret {
 func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOptions {
 	recaptchaPublicKey := component.DefaultRecaptchaPublickey()
 	recaptchaPrivateKey := component.DefaultRecaptchaPrivatekey()
-	redisSentinelHosts := component.DefaultSystemRedisSentinelHosts()
-	redisSentinelRole := component.DefaultSystemRedisSentinelRole()
-	messageBusRedisURL := component.DefaultSystemMessageBusRedisURL()
-	messageBusRedisSentinelHosts := component.DefaultSystemMessageBusRedisSentinelHosts()
-	messageBusRedisSentinelRole := component.DefaultSystemMessageBusRedisSentinelRole()
-	messageBusRedisNamespace := component.DefaultSystemMessageBusRedisNamespace()
-	redisNamespace := component.DefaultSystemRedisNamespace()
 	tmpSystemAppReplicas := int32(systemAppReplicas)
 	tmpSystemSideKiqReplicas := int32(systemSidekiqReplicas)
 	tmpSystemAdminEmail := component.DefaultSystemAdminEmail()
@@ -260,6 +305,7 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 	tmpSMTPPassword := opts.SmtpSecretOptions.Password
 	tmpSMTPPort := component.DefaultSystemSMTPPort()
 	tmpSMTPUsername := component.DefaultSystemSMTPUsername()
+	storageRequests := component.DefaultSharedStorageResources()
 
 	expectedOpts := &component.SystemOptions{
 		TenantName:                               tenantName,
@@ -288,16 +334,10 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 		ApicastAccessToken:                        opts.ApicastAccessToken,
 		AppReplicas:                               &tmpSystemAppReplicas,
 		SidekiqReplicas:                           &tmpSystemSideKiqReplicas,
-		RedisURL:                                  component.DefaultSystemRedisURL(),
-		RedisSentinelHosts:                        &redisSentinelHosts,
-		RedisSentinelRole:                         &redisSentinelRole,
-		MessageBusRedisURL:                        &messageBusRedisURL,
-		MessageBusRedisSentinelHosts:              &messageBusRedisSentinelHosts,
-		MessageBusRedisSentinelRole:               &messageBusRedisSentinelRole,
-		MessageBusRedisNamespace:                  &messageBusRedisNamespace,
-		RedisNamespace:                            &redisNamespace,
 		AdminEmail:                                &tmpSystemAdminEmail,
-		PvcFileStorageOptions:                     &component.PVCFileStorageOptions{},
+		PvcFileStorageOptions: &component.PVCFileStorageOptions{
+			StorageRequests: storageRequests,
+		},
 		SmtpSecretOptions: component.SystemSMTPSecretOptions{
 			Address:           &tmpSMTPAddress,
 			Authentication:    &tmpSMTPAuthentication,
@@ -307,19 +347,22 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 			Port:              &tmpSMTPPort,
 			Username:          &tmpSMTPUsername,
 		},
-		CommonLabels:             testSystemCommonLabels(),
-		CommonAppLabels:          testSystemCommonAppLabels(),
-		AppPodTemplateLabels:     testSystemAppPodTemplateLabels(),
-		CommonSidekiqLabels:      testSystemCommonSidekiqLabels(),
-		SidekiqPodTemplateLabels: testSystemSidekiqPodTemplateLabels(),
-		ProviderUILabels:         testSystemProviderUILabels(),
-		MasterUILabels:           testSystemMasterUILabels(),
-		DeveloperUILabels:        testSystemDeveloperUILabels(),
-		SphinxLabels:             testSystemSphinxLabels(),
-		SphinxPodTemplateLabels:  testSystemSphinxPodTemplateLabels(),
-		MemcachedLabels:          testSystemMemcachedLabels(),
-		SMTPLabels:               testSystemSMTPLabels(),
-		SideKiqMetrics:           true,
+		CommonLabels:                  testSystemCommonLabels(),
+		CommonAppLabels:               testSystemCommonAppLabels(),
+		AppPodTemplateLabels:          testSystemAppPodTemplateLabels(),
+		CommonSidekiqLabels:           testSystemCommonSidekiqLabels(),
+		SidekiqPodTemplateLabels:      testSystemSidekiqPodTemplateLabels(),
+		ProviderUILabels:              testSystemProviderUILabels(),
+		MasterUILabels:                testSystemMasterUILabels(),
+		DeveloperUILabels:             testSystemDeveloperUILabels(),
+		SphinxLabels:                  testSystemSphinxLabels(),
+		SphinxPodTemplateLabels:       testSystemSphinxPodTemplateLabels(),
+		MemcachedLabels:               testSystemMemcachedLabels(),
+		SMTPLabels:                    testSystemSMTPLabels(),
+		SideKiqMetrics:                true,
+		AppMetrics:                    true,
+		IncludeOracleOptionalSettings: true,
+		Namespace:                     opts.Namespace,
 	}
 
 	expectedOpts.ApicastSystemMasterProxyConfigEndpoint = component.DefaultApicastSystemMasterProxyConfigEndpoint(opts.ApicastAccessToken)
@@ -335,14 +378,13 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 		memcachedSecret           *v1.Secret
 		recaptchadSecret          *v1.Secret
 		eventHookSecret           *v1.Secret
-		redisSecret               *v1.Secret
 		systemAppSecret           *v1.Secret
 		systemSeedSecret          *v1.Secret
 		systemMasterApicastSecret *v1.Secret
 		expectedOptionsFactory    func(*component.SystemOptions) *component.SystemOptions
 	}{
 		{"Default", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, nil, nil, nil, nil,
+			nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				return defaultSystemOptions(opts)
 			},
@@ -352,7 +394,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager := basicApimanagerSpecTestSystemOptions()
 				apimanager.Spec.ResourceRequirementsEnabled = &falseValue
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppMasterContainerResourceRequirements = &v1.ResourceRequirements{}
@@ -364,7 +406,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithMemcachedSecret", basicApimanagerSpecTestSystemOptions,
-			getMemcachedSecret(), nil, nil, nil, nil, nil, nil,
+			getMemcachedSecret(), nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.MemcachedServers = "mymemcache:11211"
@@ -372,7 +414,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithRecaptchaSecret", basicApimanagerSpecTestSystemOptions,
-			nil, getRecaptchaSecret(), nil, nil, nil, nil, nil,
+			nil, getRecaptchaSecret(), nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				recaptchaPublicKey := "someCaptchaPK"
@@ -383,7 +425,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithEventsHookSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, getEventHookSecret(), nil, nil, nil, nil,
+			nil, nil, getEventHookSecret(), nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.BackendSharedSecret = "somePassword"
@@ -391,30 +433,8 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				return expectedOpts
 			},
 		},
-		{"WithRedisSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, getSystemRedisSecret(), nil, nil, nil,
-			func(opts *component.SystemOptions) *component.SystemOptions {
-				expectedOpts := defaultSystemOptions(opts)
-				expectedOpts.RedisURL = "redis://system1:6379"
-				redisSentinelHosts := "someHosts1"
-				expectedOpts.RedisSentinelHosts = &redisSentinelHosts
-				redisSentinelRole := "someRole1"
-				expectedOpts.RedisSentinelRole = &redisSentinelRole
-				messageBusRedisURL := "redis://system2:6379"
-				expectedOpts.MessageBusRedisURL = &messageBusRedisURL
-				messageBusRedisSentinelHosts := "someHosts2"
-				expectedOpts.MessageBusRedisSentinelHosts = &messageBusRedisSentinelHosts
-				messageBusRedisSentinelRole := "someRole2"
-				expectedOpts.MessageBusRedisSentinelRole = &messageBusRedisSentinelRole
-				redisNamespace := "systemRedis"
-				expectedOpts.RedisNamespace = &redisNamespace
-				messageBusRedisNamespace := "mbus"
-				expectedOpts.MessageBusRedisNamespace = &messageBusRedisNamespace
-				return expectedOpts
-			},
-		},
 		{"WithAppSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, nil, getSystemAppSecret(), nil, nil,
+			nil, nil, nil, getSystemAppSecret(), nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppSecretKeyBase = "somePassword234"
@@ -422,7 +442,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithSeedSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, nil, nil, getSystemSeedSecret(), nil,
+			nil, nil, nil, nil, getSystemSeedSecret(), nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.MasterName = "masterDomainName"
@@ -438,7 +458,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithMasterApicastSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, nil, nil, nil, getSystemMasterApicastSecret(),
+			nil, nil, nil, nil, nil, getSystemMasterApicastSecret(),
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.ApicastAccessToken = "apicastAccessToken"
@@ -455,7 +475,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				}
 				return apimanager
 			},
-			nil, nil, nil, nil, nil, nil, nil,
+			nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.S3FileStorageOptions = &component.S3FileStorageOptions{
@@ -469,13 +489,23 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			func() *appsv1alpha1.APIManager {
 				apimanager := basicApimanagerSpecTestSystemOptions()
 				tmp := "mystorageclassname"
-				apimanager.Spec.System.FileStorageSpec.PVC = &appsv1alpha1.SystemPVCSpec{StorageClassName: &tmp}
+				tmpVolumeName := "myvolume"
+				apimanager.Spec.System.FileStorageSpec.PVC = &appsv1alpha1.SystemPVCSpec{
+					StorageClassName: &tmp,
+					Resources: &appsv1alpha1.PersistentVolumeClaimResources{
+						Requests: resource.MustParse("456Mi"),
+					},
+					VolumeName: &tmpVolumeName,
+				}
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				tmp := "mystorageclassname"
-				expectedOpts.PvcFileStorageOptions = &component.PVCFileStorageOptions{StorageClass: &tmp}
+				tmpVolumeName := "myvolume"
+				expectedOpts.PvcFileStorageOptions.StorageClass = &tmp
+				expectedOpts.PvcFileStorageOptions.StorageRequests = resource.MustParse("456Mi")
+				expectedOpts.PvcFileStorageOptions.VolumeName = &tmpVolumeName
 				expectedOpts.S3FileStorageOptions = nil
 				return expectedOpts
 			},
@@ -487,7 +517,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager.Spec.System.SidekiqSpec.Affinity = testSystemSidekiqAffinity()
 				apimanager.Spec.System.SphinxSpec.Affinity = testSystemSphinxAffinity()
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppAffinity = testSystemAppAffinity()
@@ -503,7 +533,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager.Spec.System.SidekiqSpec.Tolerations = testSystemSidekiqTolerations()
 				apimanager.Spec.System.SphinxSpec.Tolerations = testSystemSphinxTolerations()
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppTolerations = testSystemAppTolerations()
@@ -512,6 +542,47 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				return expectedOpts
 			},
 		},
+		{"WithSystemCustomResourceRequirements",
+			func() *appsv1alpha1.APIManager {
+				apimanager := basicApimanagerSpecTestSystemOptions()
+				apimanager.Spec.System.AppSpec.MasterContainerResources = testSystemMasterContainerCustomResourceRequirements()
+				apimanager.Spec.System.AppSpec.ProviderContainerResources = testSystemProviderContainerCustomResourceRequirements()
+				apimanager.Spec.System.AppSpec.DeveloperContainerResources = testSystemDeveloperContainerCustomResourceRequirements()
+				apimanager.Spec.System.SidekiqSpec.Resources = testSystemSidekiqCustomResourceRequirements()
+				apimanager.Spec.System.SphinxSpec.Resources = testSystemSphinxCustomResourceRequirements()
+				return apimanager
+			}, nil, nil, nil, nil, nil, nil,
+			func(opts *component.SystemOptions) *component.SystemOptions {
+				expectedOpts := defaultSystemOptions(opts)
+				expectedOpts.AppMasterContainerResourceRequirements = testSystemMasterContainerCustomResourceRequirements()
+				expectedOpts.AppProviderContainerResourceRequirements = testSystemProviderContainerCustomResourceRequirements()
+				expectedOpts.AppDeveloperContainerResourceRequirements = testSystemDeveloperContainerCustomResourceRequirements()
+				expectedOpts.SidekiqContainerResourceRequirements = testSystemSidekiqCustomResourceRequirements()
+				expectedOpts.SphinxContainerResourceRequirements = testSystemSphinxCustomResourceRequirements()
+				return expectedOpts
+			},
+		},
+		{"WithSystemCustomResourceRequirementsAndGlobalResourceRequirementsDisabled",
+			func() *appsv1alpha1.APIManager {
+				apimanager := basicApimanagerSpecTestSystemOptions()
+				apimanager.Spec.ResourceRequirementsEnabled = &falseValue
+				apimanager.Spec.System.AppSpec.MasterContainerResources = testSystemMasterContainerCustomResourceRequirements()
+				apimanager.Spec.System.AppSpec.ProviderContainerResources = testSystemProviderContainerCustomResourceRequirements()
+				apimanager.Spec.System.AppSpec.DeveloperContainerResources = testSystemDeveloperContainerCustomResourceRequirements()
+				apimanager.Spec.System.SidekiqSpec.Resources = testSystemSidekiqCustomResourceRequirements()
+				apimanager.Spec.System.SphinxSpec.Resources = testSystemSphinxCustomResourceRequirements()
+				return apimanager
+			}, nil, nil, nil, nil, nil, nil,
+			func(opts *component.SystemOptions) *component.SystemOptions {
+				expectedOpts := defaultSystemOptions(opts)
+				expectedOpts.AppMasterContainerResourceRequirements = testSystemMasterContainerCustomResourceRequirements()
+				expectedOpts.AppProviderContainerResourceRequirements = testSystemProviderContainerCustomResourceRequirements()
+				expectedOpts.AppDeveloperContainerResourceRequirements = testSystemDeveloperContainerCustomResourceRequirements()
+				expectedOpts.SidekiqContainerResourceRequirements = testSystemSidekiqCustomResourceRequirements()
+				expectedOpts.SphinxContainerResourceRequirements = testSystemSphinxCustomResourceRequirements()
+				return expectedOpts
+			},
+		},
 	}
 
 	for _, tc := range cases {
@@ -526,9 +597,6 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			if tc.eventHookSecret != nil {
 				objs = append(objs, tc.eventHookSecret)
 			}
-			if tc.redisSecret != nil {
-				objs = append(objs, tc.redisSecret)
-			}
 			if tc.systemAppSecret != nil {
 				objs = append(objs, tc.systemAppSecret)
 			}
