@@ -139,6 +139,8 @@ type APIManagerCommonSpec struct {
 	ImageStreamTagImportInsecure *bool `json:"imageStreamTagImportInsecure,omitempty"`
 	// +optional
 	ResourceRequirementsEnabled *bool `json:"resourceRequirementsEnabled,omitempty"`
+	// +optional
+	ImagePullSecrets []v1.LocalObjectReference `json:"imagePullSecrets,omitempty"`
 }
 
 type ApicastSpec struct {
@@ -165,6 +167,11 @@ type ApicastProductionSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
+	// +optional
+	// +kubebuilder:validation:Minimum=1
+	Workers *int32 `json:"workers,omitempty"`
 }
 
 type ApicastStagingSpec struct {
@@ -174,6 +181,8 @@ type ApicastStagingSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type BackendSpec struct {
@@ -188,6 +197,8 @@ type BackendSpec struct {
 	// +optional
 	RedisTolerations []v1.Toleration `json:"redisTolerations,omitempty"`
 	// +optional
+	RedisResources *v1.ResourceRequirements `json:"redisResources,omitempty"`
+	// +optional
 	ListenerSpec *BackendListenerSpec `json:"listenerSpec,omitempty"`
 	// +optional
 	WorkerSpec *BackendWorkerSpec `json:"workerSpec,omitempty"`
@@ -207,6 +218,8 @@ type BackendListenerSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type BackendWorkerSpec struct {
@@ -216,6 +229,8 @@ type BackendWorkerSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type BackendCronSpec struct {
@@ -225,6 +240,8 @@ type BackendCronSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type SystemSpec struct {
@@ -238,6 +255,8 @@ type SystemSpec struct {
 	MemcachedAffinity *v1.Affinity `json:"memcachedAffinity,omitempty"`
 	// +optional
 	MemcachedTolerations []v1.Toleration `json:"memcachedTolerations,omitempty"`
+	// +optional
+	MemcachedResources *v1.ResourceRequirements `json:"memcachedResources,omitempty"`
 
 	// +optional
 	RedisImage *string `json:"redisImage,omitempty"`
@@ -248,6 +267,8 @@ type SystemSpec struct {
 	RedisAffinity *v1.Affinity `json:"redisAffinity,omitempty"`
 	// +optional
 	RedisTolerations []v1.Toleration `json:"redisTolerations,omitempty"`
+	// +optional
+	RedisResources *v1.ResourceRequirements `json:"redisResources,omitempty"`
 
 	// TODO should this field be optional? We have different approaches in Kubernetes.
 	// For example, in v1.Volume it is optional and there's an implied behaviour
@@ -279,6 +300,12 @@ type SystemAppSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	MasterContainerResources *v1.ResourceRequirements `json:"masterContainerResources,omitempty"`
+	// +optional
+	ProviderContainerResources *v1.ResourceRequirements `json:"providerContainerResources,omitempty"`
+	// +optional
+	DeveloperContainerResources *v1.ResourceRequirements `json:"developerContainerResources,omitempty"`
 }
 
 type SystemSidekiqSpec struct {
@@ -288,6 +315,8 @@ type SystemSidekiqSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type SystemSphinxSpec struct {
@@ -295,6 +324,8 @@ type SystemSphinxSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type SystemFileStorageSpec struct {
@@ -316,6 +347,13 @@ type SystemRedisPersistentVolumeClaimSpec struct {
 type SystemPVCSpec struct {
 	// +optional
 	StorageClassName *string `json:"storageClassName,omitempty"`
+	// Resources represents the minimum resources the volume should have.
+	// Ignored when VolumeName field is set
+	// +optional
+	Resources *PersistentVolumeClaimResources `json:"resources,omitempty"`
+	// VolumeName is the binding reference to the PersistentVolume backing this claim.
+	// +optional
+	VolumeName *string `json:"volumeName,omitempty"`
 }
 
 type DeprecatedSystemS3Spec struct {
@@ -349,6 +387,8 @@ type SystemMySQLSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type SystemPostgreSQLSpec struct {
@@ -361,20 +401,32 @@ type SystemPostgreSQLSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type SystemMySQLPVCSpec struct {
 	// +optional
 	StorageClassName *string `json:"storageClassName,omitempty"`
+	// Resources represents the minimum resources the volume should have.
+	// Ignored when VolumeName field is set
+	// +optional
+	Resources *PersistentVolumeClaimResources `json:"resources,omitempty"`
+	// VolumeName is the binding reference to the PersistentVolume backing this claim.
+	// +optional
+	VolumeName *string `json:"volumeName,omitempty"`
 }
 
 type SystemPostgreSQLPVCSpec struct {
 	// +optional
 	StorageClassName *string `json:"storageClassName,omitempty"`
+	// Resources represents the minimum resources the volume should have.
+	// Ignored when VolumeName field is set
 	// +optional
-	Affinity *v1.Affinity `json:"affinity,omitempty"`
+	Resources *PersistentVolumeClaimResources `json:"resources,omitempty"`
+	// VolumeName is the binding reference to the PersistentVolume backing this claim.
 	// +optional
-	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	VolumeName *string `json:"volumeName,omitempty"`
 }
 
 type ZyncSpec struct {
@@ -387,6 +439,8 @@ type ZyncSpec struct {
 	DatabaseAffinity *v1.Affinity `json:"databaseAffinity,omitempty"`
 	// +optional
 	DatabaseTolerations []v1.Toleration `json:"databaseTolerations,omitempty"`
+	// +optional
+	DatabaseResources *v1.ResourceRequirements `json:"databaseResources,omitempty"`
 
 	// +optional
 	AppSpec *ZyncAppSpec `json:"appSpec,omitempty"`
@@ -402,6 +456,8 @@ type ZyncAppSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type ZyncQueSpec struct {
@@ -411,10 +467,14 @@ type ZyncQueSpec struct {
 	Affinity *v1.Affinity `json:"affinity,omitempty"`
 	// +optional
 	Tolerations []v1.Toleration `json:"tolerations,omitempty"`
+	// +optional
+	Resources *v1.ResourceRequirements `json:"resources,omitempty"`
 }
 
 type HighAvailabilitySpec struct {
 	Enabled bool `json:"enabled,omitempty"`
+	// +optional
+	ExternalZyncDatabaseEnabled *bool `json:"externalZyncDatabaseEnabled,omitempty"`
 }
 
 type PodDisruptionBudgetSpec struct {
@@ -733,6 +793,11 @@ func (apimanager *APIManager) IsExternalDatabaseEnabled() bool {
 	return apimanager.Spec.HighAvailability != nil && apimanager.Spec.HighAvailability.Enabled
 }
 
+func (apimanager *APIManager) IsZyncExternalDatabaseEnabled() bool {
+	return apimanager.IsExternalDatabaseEnabled() &&
+		apimanager.Spec.HighAvailability.ExternalZyncDatabaseEnabled != nil &&
+		*apimanager.Spec.HighAvailability.ExternalZyncDatabaseEnabled
+}
 func (apimanager *APIManager) IsPDBEnabled() bool {
 	return apimanager.Spec.PodDisruptionBudget != nil && apimanager.Spec.PodDisruptionBudget.Enabled
 }
