@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"reflect"
 	"regexp"
+	"strings"
 
 	"github.com/3scale/3scale-operator/pkg/common"
 
@@ -95,7 +96,7 @@ type PricingRuleSpec struct {
 	MetricMethodRef MetricMethodRefSpec `json:"metricMethodRef"`
 
 	// Price per unit (USD)
-	// +kubebuilder:validation:Pattern=`^\d+.?\d{2}$`
+	// +kubebuilder:validation:Pattern=`^\d+(\.\d{2})?$`
 	PricePerUnit string `json:"pricePerUnit"`
 }
 
@@ -115,12 +116,12 @@ type ApplicationPlanSpec struct {
 	TrialPeriod *int `json:"trialPeriod,omitempty"`
 
 	// Setup fee (USD)
-	// +kubebuilder:validation:Pattern=`^\d+.?\d{2}$`
+	// +kubebuilder:validation:Pattern=`^\d+(\.\d{2})?$`
 	// +optional
 	SetupFee *string `json:"setupFee,omitempty"`
 
 	// Cost per Month (USD)
-	// +kubebuilder:validation:Pattern=`^\d+.?\d{2}$`
+	// +kubebuilder:validation:Pattern=`^\d+(\.\d{2})?$`
 	// +optional
 	CostMonth *string `json:"costMonth,omitempty"`
 
@@ -158,7 +159,7 @@ type MappingRuleSpec struct {
 	MetricMethodRef string `json:"metricMethodRef"`
 	Increment       int    `json:"increment"`
 	// +optional
-	Position *int `json:"position,omitempty"`
+	Last *bool `json:"last,omitempty"`
 }
 
 // BackendUsageSpec defines the desired state of Product's Backend Usages
@@ -765,6 +766,10 @@ type ProductStatus struct {
 	// +optional
 	State *string `json:"state,omitempty"`
 
+	// 3scale control plane host
+	// +optional
+	ProviderAccountHost string `json:"providerAccountHost,omitempty"`
+
 	// ObservedGeneration reflects the generation of the most recently observed Product Spec.
 	// +optional
 	ObservedGeneration int64 `json:"observedGeneration,omitempty"`
@@ -790,6 +795,12 @@ func (p *ProductStatus) Equals(other *ProductStatus, logger logr.Logger) bool {
 		return false
 	}
 
+	if p.ProviderAccountHost != other.ProviderAccountHost {
+		diff := cmp.Diff(p.ProviderAccountHost, other.ProviderAccountHost)
+		logger.V(1).Info("ProviderAccountHost not equal", "difference", diff)
+		return false
+	}
+
 	if p.ObservedGeneration != other.ObservedGeneration {
 		diff := cmp.Diff(p.ObservedGeneration, other.ObservedGeneration)
 		logger.V(1).Info("ObservedGeneration not equal", "difference", diff)
@@ -823,7 +834,7 @@ type Product struct {
 	Status ProductStatus `json:"status,omitempty"`
 }
 
-func (product *Product) SetDefaults() bool {
+func (product *Product) SetDefaults(logger logr.Logger) bool {
 	updated := false
 
 	// Respect 3scale API defaults
@@ -832,6 +843,14 @@ func (product *Product) SetDefaults() bool {
 		updated = true
 	}
 
+	// 3scale API ignores case of the system name field
+	systemNameLowercase := strings.ToLower(product.Spec.SystemName)
+	if product.Spec.SystemName != systemNameLowercase {
+		logger.Info("System name updated", "from", product.Spec.SystemName, "to", systemNameLowercase)
+		product.Spec.SystemName = systemNameLowercase
+		updated = true
+	}
+
 	if product.Spec.Metrics == nil {
 		product.Spec.Metrics = map[string]MetricSpec{}
 		updated = true
@@ -845,6 +864,7 @@ func (product *Product) SetDefaults() bool {
 		}
 	}
 	if !hitsFound {
+		logger.Info("Hits metric added")
 		product.Spec.Metrics["hits"] = MetricSpec{
 			Name:        "Hits",
 			Unit:        "hit",
@@ -993,6 +1013,10 @@ func (product *Product) Validate() field.ErrorList {
 	return errors
 }
 
+func (product *Product) IsSynced() bool {
+	return product.Status.Conditions.IsTrueFor(ProductSyncedConditionType)
+}
+
 func (product *Product) FindMetricOrMethod(ref string) bool {
 	if len(product.Spec.Metrics) > 0 {
 		if _, ok := product.Spec.Metrics[ref]; ok {
