@@ -6,6 +6,7 @@ import (
 	"strconv"
 
 	capabilitiesv1beta1 "github.com/3scale/3scale-operator/pkg/apis/capabilities/v1beta1"
+	controllerhelper "github.com/3scale/3scale-operator/pkg/controller/helper"
 	"github.com/3scale/3scale-operator/pkg/helper"
 	"github.com/3scale/3scale-operator/pkg/reconcilers"
 
@@ -23,35 +24,34 @@ type metricData struct {
 	spec capabilitiesv1beta1.MetricSpec
 }
 
-type mappingRuleData struct {
-	item threescaleapi.MappingRuleItem
-	spec capabilitiesv1beta1.MappingRuleSpec
-}
-
 type ThreescaleReconciler struct {
 	*reconcilers.BaseReconciler
 	backendResource     *capabilitiesv1beta1.Backend
-	backendAPIEntity    *helper.BackendAPIEntity
-	backendRemoteIndex  *helper.BackendAPIRemoteIndex
+	backendAPIEntity    *controllerhelper.BackendAPIEntity
+	backendRemoteIndex  *controllerhelper.BackendAPIRemoteIndex
 	threescaleAPIClient *threescaleapi.ThreeScaleClient
+	providerAccount     *controllerhelper.ProviderAccount
 	logger              logr.Logger
 }
 
 func NewThreescaleReconciler(b *reconcilers.BaseReconciler,
 	backendResource *capabilitiesv1beta1.Backend,
 	threescaleAPIClient *threescaleapi.ThreeScaleClient,
-	backendRemoteIndex *helper.BackendAPIRemoteIndex) *ThreescaleReconciler {
+	backendRemoteIndex *controllerhelper.BackendAPIRemoteIndex,
+	providerAccount *controllerhelper.ProviderAccount,
+) *ThreescaleReconciler {
 
 	return &ThreescaleReconciler{
 		BaseReconciler:      b,
 		backendResource:     backendResource,
 		backendRemoteIndex:  backendRemoteIndex,
 		threescaleAPIClient: threescaleAPIClient,
+		providerAccount:     providerAccount,
 		logger:              b.Logger().WithValues("3scale Reconciler", backendResource.Name),
 	}
 }
 
-func (t *ThreescaleReconciler) Reconcile() (*helper.BackendAPIEntity, error) {
+func (t *ThreescaleReconciler) Reconcile() (*controllerhelper.BackendAPIEntity, error) {
 	taskRunner := helper.NewTaskRunner(nil, t.logger)
 	taskRunner.AddTask("SyncBackend", t.syncBackend)
 	// First methods and metrics, then mapping rules.
@@ -73,7 +73,7 @@ func (t *ThreescaleReconciler) Reconcile() (*helper.BackendAPIEntity, error) {
 func (t *ThreescaleReconciler) syncBackend(_ interface{}) error {
 	var (
 		err              error
-		backendAPIEntity *helper.BackendAPIEntity
+		backendAPIEntity *controllerhelper.BackendAPIEntity
 	)
 
 	backendAPIEntity, exists := t.backendRemoteIndex.FindBySystemName(t.backendResource.Spec.SystemName)
@@ -148,7 +148,12 @@ func (t *ThreescaleReconciler) syncMethods(_ interface{}) error {
 		// notDesiredExistingKeys is a subset of the existingMap key set
 		notDesiredMap[systemName] = existingMap[systemName]
 	}
-	err = t.processNotDesiredMethods(notDesiredMap)
+	err = t.deleteNotDesiredMethodsFrom3scale(notDesiredMap)
+	if err != nil {
+		return fmt.Errorf("Error sync backend methods [%s]: %w", t.backendResource.Spec.SystemName, err)
+	}
+
+	err = t.deleteExternalMetricReferences(notDesiredExistingKeys)
 	if err != nil {
 		return fmt.Errorf("Error sync backend methods [%s]: %w", t.backendResource.Spec.SystemName, err)
 	}
@@ -205,7 +210,7 @@ func (t *ThreescaleReconciler) createNewMethods(desiredNewMap map[string]capabil
 	return nil
 }
 
-func (t *ThreescaleReconciler) processNotDesiredMethods(notDesiredMap map[string]threescaleapi.MethodItem) error {
+func (t *ThreescaleReconciler) deleteNotDesiredMethodsFrom3scale(notDesiredMap map[string]threescaleapi.MethodItem) error {
 	for _, notDesiredMethod := range notDesiredMap {
 		err := t.backendAPIEntity.DeleteMethod(notDesiredMethod.ID)
 		if err != nil {
@@ -215,6 +220,90 @@ func (t *ThreescaleReconciler) processNotDesiredMethods(notDesiredMap map[string
 	return nil
 }
 
+// valid for metrics and methods as long as 3scale ensures system_names are unique among methods and metrics
+func (t *ThreescaleReconciler) deleteExternalMetricReferences(notDesiredMetrics []string) error {
+	productList, err := controllerhelper.ProductList(t.backendResource.Namespace, t.Client(), t.providerAccount, t.logger)
+	if err != nil {
+		return fmt.Errorf("deleteExternalMetricReferences: %w", err)
+	}
+
+	// filter products referencing current backend resource
+	linkedProductList := make([]capabilitiesv1beta1.Product, 0)
+	for _, product := range productList {
+		if _, ok := product.Spec.BackendUsages[t.backendResource.Spec.SystemName]; ok {
+			linkedProductList = append(linkedProductList, product)
+		}
+	}
+
+	t.logger.V(1).Info("Product linked to backend", "total", len(linkedProductList))
+
+	for productIdx := range linkedProductList {
+		err = t.deleteExternalMetricReferencesOnProduct(notDesiredMetrics, linkedProductList[productIdx])
+		if err != nil {
+			return fmt.Errorf("deleteExternalMetricReferences: %w", err)
+		}
+	}
+
+	return nil
+}
+
+// valid for metrics and methods as long as 3scale ensures system_names are unique among methods and metrics
+func (t *ThreescaleReconciler) deleteExternalMetricReferencesOnProduct(notDesiredMetrics []string, productRef capabilitiesv1beta1.Product) error {
+	productUpdated := false
+	product := productRef.DeepCopy()
+
+	for planSystemName, planSpec := range product.Spec.ApplicationPlans {
+		planSpecUpdated := false
+
+		// Check limits with external references to the current backend
+		newLimits := make([]capabilitiesv1beta1.LimitSpec, 0)
+		for limitIdx, limitSpec := range planSpec.Limits {
+			// Check if the limit belongs to the current backend
+			// Check if the limit is marked for deletion in notDesiredMap
+			if limitSpec.MetricMethodRef.BackendSystemName == nil ||
+				*limitSpec.MetricMethodRef.BackendSystemName != t.backendResource.Spec.SystemName ||
+				!helper.ArrayContains(notDesiredMetrics, limitSpec.MetricMethodRef.SystemName) {
+				newLimits = append(newLimits, planSpec.Limits[limitIdx])
+			}
+		}
+
+		if len(newLimits) != len(planSpec.Limits) {
+			planSpecUpdated = true
+			planSpec.Limits = newLimits
+		}
+
+		// Check pricingRules with external references to the current backend
+		newRules := make([]capabilitiesv1beta1.PricingRuleSpec, 0)
+		for ruleIdx, ruleSpec := range planSpec.PricingRules {
+			// Check if the current rule belongs to the current backend
+			if ruleSpec.MetricMethodRef.BackendSystemName == nil ||
+				*ruleSpec.MetricMethodRef.BackendSystemName != t.backendResource.Spec.SystemName ||
+				!helper.ArrayContains(notDesiredMetrics, ruleSpec.MetricMethodRef.SystemName) {
+				newRules = append(newRules, planSpec.PricingRules[ruleIdx])
+			}
+		}
+
+		if len(newRules) != len(planSpec.PricingRules) {
+			planSpecUpdated = true
+			planSpec.PricingRules = newRules
+		}
+
+		if planSpecUpdated {
+			productUpdated = true
+			product.Spec.ApplicationPlans[planSystemName] = planSpec
+		}
+	}
+
+	if productUpdated {
+		err := t.UpdateResource(product)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
 func (t *ThreescaleReconciler) reconcileMatchedMethods(matchedMap map[string]methodData) error {
 	for _, data := range matchedMap {
 		params := threescaleapi.Params{}
@@ -267,7 +356,12 @@ func (t *ThreescaleReconciler) syncMetrics(_ interface{}) error {
 		// notDesiredExistingKeys is a subset of the existingMap key set
 		notDesiredMap[systemName] = existingMap[systemName]
 	}
-	err = t.processNotDesiredMetrics(notDesiredMap)
+	err = t.deleteNotDesiredMetricsFrom3scale(notDesiredMap)
+	if err != nil {
+		return fmt.Errorf("Error sync backend metrics [%s]: %w", t.backendResource.Spec.SystemName, err)
+	}
+
+	err = t.deleteExternalMetricReferences(notDesiredExistingKeys)
 	if err != nil {
 		return fmt.Errorf("Error sync backend metrics [%s]: %w", t.backendResource.Spec.SystemName, err)
 	}
@@ -327,7 +421,7 @@ func (t *ThreescaleReconciler) createNewMetrics(desiredNewMap map[string]capabil
 	return nil
 }
 
-func (t *ThreescaleReconciler) processNotDesiredMetrics(notDesiredMap map[string]threescaleapi.MetricItem) error {
+func (t *ThreescaleReconciler) deleteNotDesiredMetricsFrom3scale(notDesiredMap map[string]threescaleapi.MetricItem) error {
 	for _, metric := range notDesiredMap {
 		err := t.backendAPIEntity.DeleteMetric(metric.ID)
 		if err != nil {
@@ -372,22 +466,18 @@ func (t *ThreescaleReconciler) syncMappingRules(_ interface{}) error {
 		desiredMap[key] = spec
 	}
 
-	existingKeys := []string{}
-	existingMap := map[string]threescaleapi.MappingRuleItem{}
-	existingList, err := t.backendAPIEntity.MappingRules()
+	existingMap, err := t.getExistingMappingRules()
 	if err != nil {
 		return fmt.Errorf("Error sync backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
 	}
-	for _, item := range existingList.MappingRules {
-		key := fmt.Sprintf("%s:%s", item.Element.HTTPMethod, item.Element.Pattern)
-		existingKeys = append(existingKeys, key)
-		existingMap[key] = item.Element
+	existingKeys := make([]string, 0, len(existingMap))
+	for existingKey := range existingMap {
+		existingKeys = append(existingKeys, existingKey)
 	}
 
 	//
 	// Deleted existing and not desired mapping rules
 	//
-
 	notDesiredExistingKeys := helper.ArrayStringDifference(existingKeys, desiredKeys)
 	notDesiredList := make([]threescaleapi.MappingRuleItem, 0, len(notDesiredExistingKeys))
 	for _, key := range notDesiredExistingKeys {
@@ -400,38 +490,59 @@ func (t *ThreescaleReconciler) syncMappingRules(_ interface{}) error {
 		return fmt.Errorf("Error sync backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
 	}
 
-	//
-	// Reconcile existing and changed mapping rules
-	//
-
-	matchedKeys := helper.ArrayStringIntersection(existingKeys, desiredKeys)
-	matchedList := make([]mappingRuleData, 0, len(matchedKeys))
-	for _, key := range matchedKeys {
-		matchedList = append(matchedList, mappingRuleData{
-			item: existingMap[key],
-			spec: desiredMap[key],
-		})
+	// If existing non-desired mapping rules have been detected we refetch
+	// the existing list after deletion to get a more consistent view
+	// of the existing rules attributes after the deletion.
+	if len(notDesiredList) > 0 {
+		existingMap, err = t.getExistingMappingRules()
+		if err != nil {
+			return fmt.Errorf("Error sync backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
+		}
 	}
 
-	err = t.reconcileMatchedMappingRules(matchedList)
-	if err != nil {
-		return fmt.Errorf("Error sync backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
-	}
-
-	//
-	// Create not existing and desired mapping rules
-	//
-
-	desiredNewKeys := helper.ArrayStringDifference(desiredKeys, existingKeys)
-	desiredNewList := make([]capabilitiesv1beta1.MappingRuleSpec, 0, len(desiredNewKeys))
-	for _, key := range desiredNewKeys {
-		// key is expected to exist
-		// desiredNewKeys is a subset of the desiredKeys set
-		desiredNewList = append(desiredNewList, desiredMap[key])
-	}
-	err = t.createNewMappingRules(desiredNewList)
-	if err != nil {
-		return fmt.Errorf("Error sync backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
+	// Reconcile desired mapping rules
+	// In order of definition in the custom resource. Create or update
+	// the MappingRule being processed depending on whether it already exists
+	// in the 3scale API. Specified 'position' attribute of the MappingRule
+	// always corresponds to the position in the CR's MappingRules array.
+	// Even though when creating/updating a MappingRule the existing MappingRule
+	// positions in 3scale change, we always compare the desired keys with the
+	// existing MappingRule positions at this point. We do not refetch the list.
+	// Although this is sort of temporarily inconsistent the result is consistent
+	// because we always update positions in ascending order. The worst case of
+	// this is that some updates are performed on MappingRules whose positions
+	// are already reconciled, which is unneeded. The alternative would be to
+	// refetch all the MappingRules each time we create/update a MappingRule,
+	// which would be more inefficient than doing potential unneeded updates as
+	// it is being done with this implementation.
+	// Additionally, in case the relative position MappingRules' between
+	// unmodified MappingRules happens temporarily during the reconciliation that
+	// is not an issue due to changes are not effective until the user promotes
+	// the configuration
+	t.logger.V(1).Info("syncMappingRules", "desiredKeys", desiredKeys)
+	for desiredIdxZeroBased, desiredKey := range desiredKeys {
+		desiredMappingRule := desiredMap[desiredKey]
+		// We define the position sent to System starting from one (one-based array)
+		// instead of zero-based. The reason for that is that System API does not
+		// allow to overwrite an existing MappingRule setting it the position 0.
+		// By starting with a minimum value of position 1 we avoid that special
+		// implemented behavior by system
+		desiredIdx := desiredIdxZeroBased + 1
+		if existingMappingRule, ok := existingMap[desiredKey]; ok {
+			// Reconcile MappingRule
+			t.logger.V(1).Info("syncMappingRules", "desiredMappingRuleToReconcile", desiredKey, "position", desiredIdx)
+			err := t.reconcileMappingRuleWithPosition(desiredMappingRule, desiredIdx, existingMappingRule)
+			if err != nil {
+				return fmt.Errorf("Error sync backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
+			}
+		} else {
+			// Create MappingRule
+			t.logger.V(1).Info("syncMappingRules", "desiredMappingRuleToCreate", desiredKey, "position", desiredIdx)
+			err := t.createNewMappingRuleWithPosition(desiredMappingRule, desiredIdx)
+			if err != nil {
+				return fmt.Errorf("Error sync backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
+			}
+		}
 	}
 
 	return nil
@@ -447,68 +558,103 @@ func (t *ThreescaleReconciler) processNotDesiredMappingRules(notDesiredList []th
 	return nil
 }
 
-func (t *ThreescaleReconciler) reconcileMatchedMappingRules(matchedList []mappingRuleData) error {
-	for _, data := range matchedList {
-		params := threescaleapi.Params{}
+func (t *ThreescaleReconciler) getExistingMappingRules() (map[string]threescaleapi.MappingRuleItem, error) {
+	existingMap := map[string]threescaleapi.MappingRuleItem{}
+	existingList, err := t.backendAPIEntity.MappingRules()
+	if err != nil {
+		return nil, fmt.Errorf("Error getting backend [%s] mappingrules: %w", t.backendResource.Spec.SystemName, err)
+	}
+	for _, item := range existingList.MappingRules {
+		key := fmt.Sprintf("%s:%s", item.Element.HTTPMethod, item.Element.Pattern)
+		existingMap[key] = item.Element
+	}
 
-		//
-		// Reconcile metric or method
-		//
-		metricID, err := t.backendAPIEntity.FindMethodMetricIDBySystemName(data.spec.MetricMethodRef)
+	return existingMap, nil
+}
+
+func (t *ThreescaleReconciler) reconcileMappingRuleWithPosition(desired capabilitiesv1beta1.MappingRuleSpec, desiredPosition int, existing threescaleapi.MappingRuleItem) error {
+	params := threescaleapi.Params{}
+
+	//
+	// Reconcile metric or method
+	//
+	metricID, err := t.backendAPIEntity.FindMethodMetricIDBySystemName(desired.MetricMethodRef)
+	if err != nil {
+		return fmt.Errorf("Error reconcile backend mapping rule: %w", err)
+	}
+
+	if metricID < 0 {
+		// Should not happen as metric and method references have been validated and should exists
+		return errors.New("backend metric method ref for mapping rule not found")
+	}
+
+	if metricID != existing.MetricID {
+		params["metric_id"] = strconv.FormatInt(metricID, 10)
+	}
+
+	//
+	// Reconcile delta
+	//
+	if desired.Increment != existing.Delta {
+		params["delta"] = strconv.Itoa(desired.Increment)
+	}
+
+	//
+	// Reconcile last
+	//
+	desiredLastAttribute := false
+	if desired.Last != nil {
+		desiredLastAttribute = *desired.Last
+	}
+
+	if desiredLastAttribute != existing.Last {
+		params["last"] = strconv.FormatBool(desiredLastAttribute)
+	}
+
+	//
+	// Reconcile Position
+	//
+	if desiredPosition != existing.Position {
+		params["position"] = strconv.FormatInt(int64(desiredPosition), 10)
+	}
+
+	if len(params) > 0 {
+		err := t.backendAPIEntity.UpdateMappingRule(existing.ID, params)
 		if err != nil {
 			return fmt.Errorf("Error reconcile backend mapping rule: %w", err)
 		}
-
-		if metricID < 0 {
-			// Should not happen as metric and method references have been validated and should exists
-			return errors.New("backend metric method ref for mapping rule not found")
-		}
-
-		if metricID != data.item.MetricID {
-			params["metric_id"] = strconv.FormatInt(metricID, 10)
-		}
-
-		//
-		// Reconcile delta
-		//
-		if data.spec.Increment != data.item.Delta {
-			params["delta"] = strconv.Itoa(data.spec.Increment)
-		}
-
-		if len(params) > 0 {
-			err := t.backendAPIEntity.UpdateMappingRule(data.item.ID, params)
-			if err != nil {
-				return fmt.Errorf("Error reconcile backend mapping rule: %w", err)
-			}
-		}
 	}
 
 	return nil
 }
 
-func (t *ThreescaleReconciler) createNewMappingRules(desiredList []capabilitiesv1beta1.MappingRuleSpec) error {
-	for _, spec := range desiredList {
-		metricID, err := t.backendAPIEntity.FindMethodMetricIDBySystemName(spec.MetricMethodRef)
-		if err != nil {
-			return fmt.Errorf("Error creating backend [%s] mappingrule: %w", t.backendResource.Spec.SystemName, err)
-		}
+func (t *ThreescaleReconciler) createNewMappingRuleWithPosition(desired capabilitiesv1beta1.MappingRuleSpec, desiredPosition int) error {
+	metricID, err := t.backendAPIEntity.FindMethodMetricIDBySystemName(desired.MetricMethodRef)
+	if err != nil {
+		return fmt.Errorf("Error creating backend [%s] mappingrule: %w", t.backendResource.Spec.SystemName, err)
+	}
 
-		if metricID < 0 {
-			// Should not happen as metric and method references have been validated and should exists
-			return errors.New("backend metric method ref for mapping rule not found")
-		}
+	if metricID < 0 {
+		// Should not happen as metric and method references have been validated and should exists
+		return errors.New("backend metric method ref for mapping rule not found")
+	}
 
-		params := threescaleapi.Params{
-			"pattern":     spec.Pattern,
-			"http_method": spec.HTTPMethod,
-			"metric_id":   strconv.FormatInt(metricID, 10),
-			"delta":       strconv.Itoa(spec.Increment),
-		}
+	params := threescaleapi.Params{
+		"pattern":     desired.Pattern,
+		"http_method": desired.HTTPMethod,
+		"metric_id":   strconv.FormatInt(metricID, 10),
+		"delta":       strconv.Itoa(desired.Increment),
+	}
 
-		err = t.backendAPIEntity.CreateMappingRule(params)
-		if err != nil {
-			return fmt.Errorf("Error creating backend [%s] mappingrule: %w", t.backendResource.Spec.SystemName, err)
-		}
+	if desired.Last != nil {
+		params["last"] = strconv.FormatBool(*desired.Last)
+	}
+
+	params["position"] = strconv.FormatInt(int64(desiredPosition), 10)
+
+	err = t.backendAPIEntity.CreateMappingRule(params)
+	if err != nil {
+		return fmt.Errorf("Error creating backend [%s] mappingrule: %w", t.backendResource.Spec.SystemName, err)
 	}
 	return nil
 }
